package ink.wgink.common.base;

import ink.wgink.interfaces.menu.IMenuBaseService;
import ink.wgink.interfaces.user.IUserCheckService;
import ink.wgink.pojo.dtos.menu.MenuDTO;
import ink.wgink.util.map.HashMapUtil;
import ink.wgink.util.request.RequestUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: AbstractController
 * @Description: baseinfo controller
 * @Author: wenc
 * @Date: 2018/9/27 下午3:47
 * @Version: 1.0
 **/
@Controller
public class DefaultBaseController {

    protected static final Logger LOG = LoggerFactory.getLogger(DefaultBaseController.class);

    @Autowired
    private HttpSession httpSession;
    @Autowired(required = false)
    private IMenuBaseService menuBaseService;
    @Autowired(required = false)
    private IUserCheckService userCheckService;

    @GetMapping("index")
    public ModelAndView goIndex() {
        ModelAndView mv = new ModelAndView("index");

        if (menuBaseService != null) {
            List<MenuDTO> menus;
            // 加载菜单
            if (userCheckService != null) {
                // 引入用户模块，是统一用户系统，加载统一用户系统菜单
                menus = menuBaseService.listAllByParentId(IMenuBaseService.MENU_UNIFIED_USER);
            } else {
                // 未引入用户模块，是客户端系统，加载客户端菜单
                menus = new ArrayList<>();
            }

            mv.addObject("menus", menus);
        }

        return mv;
    }

    /**
     * 获取请求参数
     *
     * @return
     */
    protected Map<String, Object> requestParams() {
        Map<String, Object> params = requestParams(RequestUtil.getRequest());
        String keywords = "keywords";
        if (!StringUtils.isEmpty(params.get(keywords))) {
            try {
                params.replace(keywords, URLDecoder.decode(params.get(keywords).toString().trim(), "UTF-8"));
            } catch (Exception e) {
                LOG.error(e.getMessage(), e);
            }
        }
        String showLevel = "showLevel";
        if (StringUtils.isEmpty(params.get(showLevel))) {
            params.put(showLevel, "0");
        }
        return params;
    }

    /**
     * 获取请求参数
     *
     * @return
     */
    protected Map<String, Object> requestParams(HttpServletRequest request) {
        Map<String, Object> params = HashMapUtil.requestParamsToMap(request);
        params.put("requestUri", request.getRequestURI());
        params.put("requestHost", request.getRemoteHost());
        params.put("requestPort", request.getLocalPort());
        return params;
    }

    /**
     * 获取参数
     *
     * @return
     */
    protected Map<String, Object> getParams() {
        return new HashMap<>(0);
    }

    /**
     * 获取session
     *
     * @return
     */
    protected HttpSession getHttpSession() {
        return httpSession;
    }
}
