package ink.wgink.module.permission.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.permission.PermissionDTO;
import ink.wgink.pojo.result.SuccessResultData;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.module.permission.dao.IPermissionDao;
import ink.wgink.module.permission.pojo.vos.PermissionVO;
import ink.wgink.module.permission.service.IPermissionService;
import ink.wgink.util.UUIDUtil;
import ink.wgink.util.map.HashMapUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: PermissionServiceImpl
 * @Description: 权限
 * @Author: WenG
 * @Date: 2020-05-27 15:58
 * @Version: 1.0
 **/
@Service
public class PermissionServiceImpl extends DefaultBaseService implements IPermissionService {

    @Autowired
    private IPermissionDao permissionDao;

    @Override
    public void save(PermissionVO permissionVO) {
        savePermissionReturnId(permissionVO);
    }

    @Override
    public String savePermissionReturnId(PermissionVO permissionVO) {
        String permissionId = UUIDUtil.getUUID();
        Map<String, Object> params = HashMapUtil.beanToMap(permissionVO);
        params.put("permissionId", permissionId);
        setSaveInfo(params);
        permissionDao.save(params);
        return permissionId;
    }

    @Override
    public void remove(List<String> ids) {
        Map<String, Object> params = getHashMap(3);
        params.put("permissionIds", ids);
        setUpdateInfo(params);
        permissionDao.remove(params);
    }

    @Override
    public void deletePermission(List<String> ids) {
        Map<String, Object> params = getHashMap(3);
        params.put("permissionIds", ids);
        permissionDao.delete(params);
    }

    @Override
    public void update(String permissionId, PermissionVO permissionVO) {
        Map<String, Object> params = HashMapUtil.beanToMap(permissionVO);
        params.put("permissionId", permissionId);
        setUpdateInfo(params);
        permissionDao.update(params);
    }

    @Override
    public PermissionDTO get(String permissionId) {
        Map<String, Object> params = super.getHashMap(1);
        params.put("permissionId", permissionId);
        return permissionDao.get(params);
    }

    @Override
    public List<PermissionDTO> list(Map<String, Object> params) {
        return permissionDao.list(params);
    }

    @Override
    public SuccessResultList<List<PermissionDTO>> listPage(ListPage page) {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<PermissionDTO> permissionDTOs = permissionDao.list(page.getParams());
        PageInfo<PermissionDTO> pageInfo = new PageInfo<>(permissionDTOs);
        return new SuccessResultList<>(permissionDTOs, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    @Override
    public Integer countNumberPermission(Map<String, Object> params) {
        Integer count = permissionDao.count(params);
        return count == null ? 0 : count;
    }

    @Override
    public SuccessResultData<Integer> count(Map<String, Object> params) {
        return new SuccessResultData<>(countNumberPermission(params));
    }

    @Override
    public List<PermissionDTO> listByIds(List<String> permissionIds) {
        Map<String, Object> params = getHashMap(2);
        params.put("permissionIds", permissionIds);
        return list(params);
    }

}