package ink.wgink.service.department.service.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SearchException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.interfaces.user.IUserBaseService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.department.DepartmentSimpleDTO;
import ink.wgink.pojo.dtos.department.DepartmentUserDTO;
import ink.wgink.pojo.dtos.user.UserDTO;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.service.department.dao.IDepartmentUserDao;
import ink.wgink.service.department.service.IDepartmentAdjustmentService;
import ink.wgink.service.department.service.IDepartmentService;
import ink.wgink.service.department.service.IDepartmentUserService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: DepartmentUserServiceImpl
 * @Description: 组织部门用户
 * @Author: wanggeng
 * @Date: 2021/1/28 4:21 下午
 * @Version: 1.0
 */
@Service
public class DepartmentUserServiceImpl extends DefaultBaseService implements IDepartmentUserService {

    @Autowired
    private IDepartmentUserDao departmentUserDao;
    @Autowired
    private IDepartmentService departmentService;
    @Autowired
    private IDepartmentAdjustmentService departmentAdjustmentService;
    @Autowired
    private IUserBaseService userBaseService;

    @Override
    public void save(String departmentId, String departmentName, List<String> departmentUserIds) {
        Map<String, Object> params = getHashMap(2);
        for (String departmentUserId : departmentUserIds) {
            params.put("userId", departmentUserId);
            params.put("departmentId", departmentId);
            departmentUserDao.save(params);
        }
        departmentAdjustmentService.save(departmentUserIds, departmentId, departmentName, USER_DEPARTMENT_TYPE_JOIN);
    }

    @Override
    public void delete(String departmentId, String departmentName, List<String> userIds) {
        String adjustmentDepartmentName = departmentName;
        if (StringUtils.isBlank(departmentName)) {
            DepartmentSimpleDTO departmentSimpleDTO = departmentService.getSimple(departmentId);
            if (departmentSimpleDTO == null) {
                throw new SearchException("部门不存在");
            }
            adjustmentDepartmentName = departmentSimpleDTO.getDepartmentName();
        }
        if (userIds == null || userIds.isEmpty()) {
            return;
        }
        Map<String, Object> params = getHashMap(1);
        params.put("departmentId", departmentId);
        params.put("userIds", userIds);
        departmentUserDao.delete(params);
        departmentAdjustmentService.save(userIds, departmentId, adjustmentDepartmentName, USER_DEPARTMENT_TYPE_LEAVE);
    }

    @Override
    public void delete(String departmentId) {
        Map<String, Object> params = getHashMap(2);
        params.put("departmentId", departmentId);
        List<String> userIds = departmentUserDao.listUserId(params);
        delete(departmentId, null, userIds);
    }

    @Override
    public void update(String departmentId, List<String> userIds) {
        if (userIds.isEmpty()) {
            return;
        }
        DepartmentSimpleDTO departmentSimpleDTO = departmentService.getSimple(departmentId);
        if (departmentSimpleDTO == null) {
            throw new SearchException("部门不存在");
        }
        Map<String, Object> params = getHashMap(2);
        params.put("departmentId", departmentId);
        List<String> existUserIds = departmentUserDao.listUserId(params);
        List<String> newUserIds = new ArrayList<>();
        for (String userId : userIds) {
            boolean isNewUser = true;
            for (String existUserId : existUserIds) {
                if (StringUtils.equals(userId, existUserId)) {
                    isNewUser = false;
                    break;
                }
            }
            if (isNewUser) {
                params.put("userId", userId);
                departmentUserDao.save(params);
                newUserIds.add(userId);
            }
        }
        departmentAdjustmentService.save(newUserIds, departmentId, departmentSimpleDTO.getDepartmentName(), USER_DEPARTMENT_TYPE_JOIN);
    }

    @Override
    public List<String> listUserId(String departmentId) {
        Map<String, Object> params = getHashMap(2);
        params.put("departmentId", departmentId);
        return departmentUserDao.listUserId(params);
    }

    @Override
    public List<String> listUserId(List<String> departmentIds) {
        Map<String, Object> params = getHashMap(2);
        params.put("departmentIds", departmentIds);
        return departmentUserDao.listUserId(params);
    }

    @Override
    public SuccessResultList<List<DepartmentUserDTO>> listPage(String departmentId, ListPage page) {
        SuccessResultList<List<UserDTO>> successResultList;
        if (StringUtils.equals(departmentId, ISystemConstant.TREE_BASE_ROOT_ID_VALUE)) {
            successResultList = listPageUserByExcludeDepartment(page);
        } else {
            successResultList = listPageUser(departmentId, page);
        }
        List<UserDTO> userDTOs = successResultList.getRows();
        List<DepartmentUserDTO> departmentUserDTOs = new ArrayList<>();
        for (UserDTO userDTO : userDTOs) {
            DepartmentUserDTO departmentUserDTO = new DepartmentUserDTO(userDTO);
            departmentUserDTO.setDepartmentId(departmentId);
            departmentUserDTOs.add(departmentUserDTO);
        }
        return new SuccessResultList<>(departmentUserDTOs, successResultList.getPage(), successResultList.getTotal());
    }

    private List<String> listGroupUserId(Map<String, Object> params) {
        return departmentUserDao.listGroupUserId(params);
    }

    /**
     * 没有组织机构的用户分页列表
     *
     * @param page
     * @return
     */
    private SuccessResultList<List<UserDTO>> listPageUserByExcludeDepartment(ListPage page) {
        List<String> hasDepartmentUserIds = listGroupUserId(new HashMap<>(0));
        return userBaseService.listPageByExcludeIds(hasDepartmentUserIds, page);
    }

    /**
     * 组织机构的用户分页列表
     *
     * @param departmentId 组织机构ID
     * @param page
     * @return
     */
    private SuccessResultList<List<UserDTO>> listPageUser(String departmentId, ListPage page) {
        List<String> userIds = listUserId(departmentId);
        if (userIds.isEmpty()) {
            return new SuccessResultList<>(new ArrayList<>(), 1, 0L);
        }
        return userBaseService.listPageByIds(userIds, page);
    }
}
