package ink.wgink.service.role.controller.api;

import ink.wgink.annotation.CheckRequestBodyAnnotation;
import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.exceptions.*;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.role.RoleDTO;
import ink.wgink.pojo.dtos.role.RoleDataAuthorityTypeDTO;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.service.role.pojo.vos.RoleDataAuthorityVO;
import ink.wgink.service.role.pojo.vos.RoleUserAuthorizationVO;
import ink.wgink.service.role.pojo.vos.RoleVO;
import ink.wgink.service.role.service.IRoleService;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: RoleController
 * @Description: 角色
 * @Author: wenc
 * @Date: 2019/1/6 3:03 PM
 * @Version: 1.0
 * @MenuUrl: admin/role/goroles
 **/
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "角色信息管理接口")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/role")
public class RoleController extends DefaultBaseController {

    @Autowired
    private IRoleService roleService;
//    @Autowired
//    private IMenuService menuService;

    @ApiOperation(value = "角色新增", notes = "角色新增接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("saverole")
    public SuccessResult saveRole(@RequestBody RoleVO roleVO) {
        Map<String, Object> params = getParams();
        checkParams(roleVO, params);
        return roleService.saveRole(params);
    }

    @ApiOperation(value = "角色删除", notes = "通过id列表批量删除角色接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "角色ID列表，用下划线分隔", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("removerole/{ids}")
    public SuccessResult removeRole(@PathVariable("ids") String ids) throws RemoveException {
        Map<String, Object> params = getParams();
        params.put("roleIds", ids);
        return roleService.removeRole(params);
    }

    @ApiOperation(value = "角色修改", notes = "角色修改接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roleId", value = "角色ID", paramType = "path"),
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("updaterole/{roleId}")
    public SuccessResult updateRole(@PathVariable("roleId") String roleId, @RequestBody RoleVO roleVO) {
        Map<String, Object> params = getParams();
        params.put("roleId", roleId);
        checkParams(roleVO, params);
        return roleService.updateRole(params);
    }

    /**
     * 参数校验
     *
     * @param roleVO
     * @param params
     * @throws ParamsException
     */
    private void checkParams(RoleVO roleVO, Map<String, Object> params) {
        if (StringUtils.isBlank(roleVO.getRoleParentId())) {
            throw new ParamsException("上级ID不能为空");
        }
        params.put("roleParentId", roleVO.getRoleParentId());

        if (StringUtils.isBlank(roleVO.getRoleName())) {
            throw new ParamsException("角色名称不能为空");
        }
        params.put("roleName", roleVO.getRoleName());

        params.put("roleSummary", roleVO.getRoleSummary());
    }

    @ApiOperation(value = "角色列表", notes = "角色列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roleParentId", value = "角色上级ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listrolesalljson/{roleParentId}")
    public List<RoleDTO> listRoles(@PathVariable("roleParentId") String roleParentId) {
        Map<String, Object> params = getParams();
        params.put("roleParentId", roleParentId);
        return roleService.listRolesAllJson(params);
    }

    @ApiOperation(value = "角色详情", notes = "角色详情接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roleId", value = "角色ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("getrole/{roleId}")
    public RoleDTO getRole(@PathVariable("roleId") String roleId) {
        Map<String, Object> params = getParams();
        params.put("roleId", roleId);
        return roleService.getRole(params);
    }

    @ApiOperation(value = "zTree列表", notes = "zTree列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "上级ID", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listztreerole")
    public List<ZTreeDTO> listZTreeRole() throws SearchException {
        Map<String, Object> params = requestParams();
        String roleParentId = "0";
        if (!StringUtils.isBlank(params.get("id") == null ? null : params.get("id").toString())) {
            roleParentId = params.get("id").toString();
        }
        params.put("roleParentId", roleParentId);
        return roleService.listZTreeRole(params);
    }

    @ApiOperation(value = "easyui角色列表", notes = "easyui角色列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "parentId", value = "上级ID", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpagerole")
    public SuccessResultList<List<RoleDTO>> listPageRole(ListPage page) throws SearchException {
        Map<String, Object> params = requestParams();
        String roleParentId = "0";
        if (!StringUtils.isBlank(params.get("parentId") == null ? null : params.get("parentId").toString())) {
            roleParentId = params.get("parentId").toString();
        }
        params.put("roleParentId", roleParentId);
        page.setParams(params);
        return roleService.listPageRole(page);
    }

//    @ApiOperation(value = "角色菜单列表", notes = "角色菜单列表接口")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "menuParentId", value = "上级菜单ID", paramType = "path", dataType = "String"),
//            @ApiImplicitParam(name = "roleId", value = "角色ID", paramType = "path", dataType = "String"),
//            @ApiImplicitParam(name = "type", value = "类型", paramType = "path", dataType = "String")
//    })
//    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
//    @GetMapping("listmenu/{menuParentId}/{roleId}/{type}")
//    public List<MenuDTO> listMenu(@PathVariable("menuParentId") String menuParentId,
//                                  @PathVariable("roleId") String roleId,
//                                  @PathVariable("type") String type) throws SearchException, ParamsException {
//        Map<String, Object> params = requestParams();
//        params.put("menuParentId", menuParentId);
//        params.put("roleId", roleId);
//        if (!IRoleService.AUTH_TYPE_INSERT.equals(type) &&
//                !IRoleService.AUTH_TYPE_DELETE.equals(type) &&
//                !IRoleService.AUTH_TYPE_UPDATE.equals(type) &&
//                !IRoleService.AUTH_TYPE_QUERY.equals(type) &&
//                !IRoleService.AUTH_TYPE_MENU.equals(type)) {
//            throw new ParamsException("授权类型错误");
//        }
//        params.put("type", type);
//        if (IRoleService.AUTH_TYPE_MENU.equals(type)) {
//            params.put("menuType", 1);
//        }
//        return menuService.listMenuAllJson(params);
//    }

    @ApiOperation(value = "执行授权", notes = "执行授权接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roleId", value = "角色ID", paramType = "path", dataType = "String"),
            @ApiImplicitParam(name = "type", value = "类型", paramType = "path", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("authorization/{roleId}/{type}")
    public SuccessResult authorization(@PathVariable("roleId") String roleId,
                                       @PathVariable("type") String type,
                                       @RequestBody RoleUserAuthorizationVO roleUserAuthorizationVO) {
        Map<String, Object> params = getParams();
        params.put("roleId", roleId);
        if (!IRoleService.AUTH_TYPE_INSERT.equals(type) &&
                !IRoleService.AUTH_TYPE_DELETE.equals(type) &&
                !IRoleService.AUTH_TYPE_UPDATE.equals(type) &&
                !IRoleService.AUTH_TYPE_QUERY.equals(type) &&
                !IRoleService.AUTH_TYPE_MENU.equals(type)) {
            throw new ParamsException("授权类型错误");
        }
        params.put("type", type);
        params.put("menuIds", roleUserAuthorizationVO.getMenuIds());
        return roleService.authorization(params);
    }

    @ApiOperation(value = "更新角色数据权限", notes = "更新角色数据权限接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("updateroledataauthority/{roleId}")
    @CheckRequestBodyAnnotation
    public SuccessResult updateRoleDataAuthority(@PathVariable("roleId") String roleId, @RequestBody RoleDataAuthorityVO roleDataAuthorityVO) {
        return roleService.updateRoleDataAuthority(roleId, roleDataAuthorityVO);
    }

    @ApiOperation(value = "获取角色数据权限类型", notes = "获取角色数据权限类型接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("getroledataauthoritytype/{roleId}")
    public RoleDataAuthorityTypeDTO getRoleDataAuthorityType(@PathVariable("roleId") String roleId) {
        return roleService.getRoleDataAuthorityType(roleId);
    }

}
