package ink.wgink.service.role.dao;

import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SaveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.exceptions.UpdateException;
import ink.wgink.service.role.pojo.pos.RolePO;
import ink.wgink.pojo.bos.RoleBO;
import ink.wgink.pojo.bos.RoleMenuBO;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.role.RoleDTO;
import ink.wgink.pojo.dtos.role.RoleDataAuthorityDTO;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IRoleDao
 * @Description: 角色
 * @Author: WangGeng
 * @Date: 2021/1/31 18:52
 * @Version: 1.0
 **/
@Repository
public interface IRoleDao {

    /**
     * 新增角色
     *
     * @param params
     * @throws SaveException
     */
    void saveRole(Map<String, Object> params) throws SaveException;

    /**
     * 删除角色
     *
     * @param params
     * @throws RemoveException
     */
    void removeRole(Map<String, Object> params) throws RemoveException;

    /**
     * 修改角色
     *
     * @param params
     * @throws UpdateException
     */
    void updateRole(Map<String, Object> params) throws UpdateException;

    /**
     * 角色列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RoleDTO> listRole(Map<String, Object> params) throws SearchException;

    /**
     * 角色详情
     *
     * @param params
     * @return
     * @throws SearchException
     */
    RoleDTO getRole(Map<String, Object> params) throws SearchException;

    /**
     * 角色详情
     *
     * @param params
     * @return
     * @throws SearchException
     */
    RolePO getRolePO(Map<String, Object> params) throws SearchException;

    /**
     * 获取最后一个字角色
     *
     * @param parentId
     * @return
     * @throws SearchException
     */
    RoleDTO getLastByParentId(String parentId) throws SearchException;

    /**
     * 角色ZTree列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<ZTreeDTO> listZTreeRoles(Map<String, Object> params) throws SearchException;

    /**
     * 子列表个数
     *
     * @param id
     * @return
     * @throws SearchException
     */
    Integer countByParentId(String id) throws SearchException;

    /**
     * 通过用户获取角色
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RoleBO> listRoleBOByUser(Map<String, Object> params) throws SearchException;

    /**
     * 通过用户组获取角色
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RoleBO> listRoleBOByGroup(Map<String, Object> params) throws SearchException;

    /**
     * 删除角色菜单信息
     *
     * @param params
     * @throws RemoveException
     */
    void removeRoleMenuInfo(Map<String, Object> params) throws RemoveException;

    /**
     * 保存角色菜单信息
     *
     * @param params
     * @throws SaveException
     */
    void saveRoleMenuInfo(Map<String, Object> params) throws SaveException;

    /**
     * 角色菜单列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RoleMenuBO> listRoleMenuInfo(Map<String, Object> params) throws SearchException;

    /**
     * 角色菜单列表详情
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RoleMenuBO> listRoleMenuDetail(Map<String, Object> params) throws SearchException;

    /**
     * 新增自定义角色数据权限
     *
     * @param params
     * @throws SaveException
     */
    void saveRoleDataAuthority(Map<String, Object> params) throws SaveException;

    /**
     * 删除自定义角色数据权限
     *
     * @param params
     * @throws RemoveException
     */
    void deleteRoleDataAuthority(Map<String, Object> params) throws RemoveException;

    /**
     * 角色权限列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RoleDataAuthorityDTO> listRoleDataAuthority(Map<String, Object> params) throws SearchException;

    /**
     * 统计角色
     *
     * @param params
     * @return
     * @throws SearchException
     */
    Integer countRole(Map<String, Object> params) throws SearchException;

}
