package ink.wgink.service.role.service;

import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SaveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.exceptions.UpdateException;
import ink.wgink.interfaces.role.IRoleBaseService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.bos.DepartmentBO;
import ink.wgink.pojo.bos.GroupBO;
import ink.wgink.pojo.bos.RoleBO;
import ink.wgink.pojo.bos.RoleMenuBO;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.role.RoleDTO;
import ink.wgink.pojo.dtos.role.RoleDataAuthorityDTO;
import ink.wgink.pojo.dtos.role.RoleDataAuthorityTypeDTO;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.service.role.pojo.vos.RoleDataAuthorityVO;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IRoleService
 * @Description: 角色
 * @Author: WangGeng
 * @Date: 2021/1/31 18:50
 * @Version: 1.0
 **/
public interface IRoleService extends IRoleBaseService {

    /**
     * 授权新增
     */
    String AUTH_TYPE_INSERT = "1";
    /**
     * 授权删除
     */
    String AUTH_TYPE_DELETE = "2";
    /**
     * 授权更新
     */
    String AUTH_TYPE_UPDATE = "3";
    /**
     * 授权查询
     */
    String AUTH_TYPE_QUERY = "4";
    /**
     * 授权菜单
     */
    String AUTH_TYPE_MENU = "5";
    /**
     * 角色菜单新增
     */
    String ROLE_INSERT = "roleInsert";
    /**
     * 角色菜单删除
     */
    String ROLE_DELETE = "roleDelete";
    /**
     * 角色菜单更新
     */
    String ROLE_UPDATE = "roleUpdate";
    /**
     * 角色菜单查询
     */
    String ROLE_QUERY = "roleQuery";
    /**
     * 角色菜单
     */
    String ROLE_MENU = "roleMenu";
    /**
     * 角色类型
     */
    String ROLE_TYPE = "roleType";

    /**
     * 新增角色
     *
     * @param params
     * @return
     * @throws SaveException
     * @throws SearchException
     */
    SuccessResult saveRole(Map<String, Object> params);

    /**
     * 删除角色
     *
     * @param params
     * @return
     * @throws RemoveException
     */
    SuccessResult removeRole(Map<String, Object> params);

    /**
     * 修改角色
     *
     * @param params
     * @return
     * @throws UpdateException
     */
    SuccessResult updateRole(Map<String, Object> params);

    /**
     * 全部角色JSON列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RoleDTO> listRolesAllJson(Map<String, Object> params);

    /**
     * 角色详情
     *
     * @param params
     * @return
     */
    RoleDTO getRole(Map<String, Object> params);

    /**
     * 角色列表，递归获取全部内容
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RoleDTO> listRolesAll(Map<String, Object> params);

    /**
     * 角色zTree列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<ZTreeDTO> listZTreeRole(Map<String, Object> params);

    /**
     * 角色EasyUI列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<RoleDTO>> listPageRole(ListPage page);

    /**
     * 执行授权
     *
     * @param params
     * @return
     * @throws UpdateException
     */
    SuccessResult authorization(Map<String, Object> params);

    /**
     * 通过用户获取角色
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RoleBO> listRoleBOByUser(Map<String, Object> params);

    /**
     * 通过用户组获取角色
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RoleBO> listRoleBOByGroup(Map<String, Object> params);

    /**
     * 角色菜单列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RoleMenuBO> listRoleMenuDetail(Map<String, Object> params);

    /**
     * 通过roleId获取角色
     *
     * @param roleId
     * @return
     * @throws SearchException
     */
    public RoleBO getRoleBOByRoleId(String roleId);

    /**
     * 获取角色权限集合
     *
     * @param roleIds
     * @return
     * @throws SearchException
     */
    List<RoleBO> listRoleBO(String roleIds);

    /**
     * 更新角色数据权限
     *
     * @param roleId
     * @param roleDataAuthorityVO
     * @return
     * @throws SaveException
     * @throws RemoveException
     */
    SuccessResult updateRoleDataAuthority(String roleId, RoleDataAuthorityVO roleDataAuthorityVO);

    /**
     * 获取角色数据权限类型
     *
     * @param roleId
     * @return
     * @throws SearchException
     */
    RoleDataAuthorityTypeDTO getRoleDataAuthorityType(String roleId);

    /**
     * 获取角色数据权限列表（通过ID列表）
     *
     * @param roleIds
     * @return
     * @throws SearchException
     */
    List<RoleDataAuthorityDTO> listRoleDataAuthorityByRoleIdList(List<String> roleIds);

    /**
     * 通过角色列表获取数据权限
     *
     * @param roleBOs
     * @return
     */
    List<String> listDataAuthorityUserIdByRole(List<RoleBO> roleBOs);

    /**
     * 通过部门获取用户数据数据权限
     *
     * @param departmentBOs
     * @return
     */
    List<String> listDataAuthorityUserIdByDepartment(List<DepartmentBO> departmentBOs);

    /**
     * 获取角色数据权限
     *
     * @param roleBOs
     * @return
     */
    String getRoleDataAuthority(List<RoleBO> roleBOs);

    /**
     * 设置角色菜单
     *
     * @param roleBO
     * @throws SearchException
     */
    void setRoleMenu(RoleBO roleBO);

    /**
     * 批量设置角色菜单
     *
     * @param roleBOs
     * @throws SearchException
     */
    void setRoleMenus(List<RoleBO> roleBOs);

    /**
     * 设置组角色列表
     *
     * @param groupRoles
     * @param roleBOs
     */
    void setGroupRoles(List<GroupBO> groupRoles, List<RoleBO> roleBOs);

    /**
     * 去除重复值
     *
     * @param roleBOs
     */
    void removalDuplicateRole(List<RoleBO> roleBOs);

    /**
     * 统计角色
     *
     * @param params
     * @return
     * @throws SearchException
     */
    Integer countRole(Map<String, Object> params);

}
