package ink.wgink.service.role.service.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.pojo.dtos.permission.PermissionDTO;
import ink.wgink.pojo.dtos.role.RolePermissionDTO;
import ink.wgink.module.permission.service.IPermissionService;
import ink.wgink.service.role.dao.IRolePermissionDao;
import ink.wgink.service.role.pojo.vos.RolePermissionVO;
import ink.wgink.service.role.service.IRolePermissionService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: RolePermissionServiceImpl
 * @Description: 角色权限
 * @Author: WangGeng
 * @Date: 2021/1/31 21:23
 * @Version: 1.0
 **/
@Service
public class RolePermissionServiceImpl extends DefaultBaseService implements IRolePermissionService {

    @Autowired
    private IRolePermissionDao rolePermissionDao;
    @Autowired
    private IPermissionService permissionService;

    @Override
    public void updateByRoleId(String roleId, RolePermissionVO rolePermissionVO) {
        Map<String, Object> params = getHashMap(2);
        params.put("roleId", roleId);
        if (rolePermissionVO.getChecked()) {
            LOG.debug("新增权限角色，roleId: {}", roleId);
            for (String permissionId : rolePermissionVO.getPermissionIdArray()) {
                if (getByPermissionIdAndRoleId(permissionId, roleId) != null) {
                    continue;
                }
                params.put("permissionId", permissionId);
                rolePermissionDao.save(params);
            }
        } else {
            LOG.debug("删除权限角色，roleId: {}", roleId);
            params.put("permissionIds", rolePermissionVO.getPermissionIdArray());
            rolePermissionDao.delete(params);
        }
    }

    @Override
    public RolePermissionDTO getByPermissionIdAndRoleId(String permissionId, String roleId) {
        PermissionDTO permissionDTO = permissionService.get(permissionId);
        if (permissionDTO == null) {
            return null;
        }
        Map<String, Object> params = getHashMap(2);
        params.put("permissionId", permissionId);
        params.put("roleId", roleId);
        RolePermissionDTO rolePermissionDTO = rolePermissionDao.get(params);
        if (rolePermissionDTO == null) {
            return null;
        }
        rolePermissionDTO.setPermissionTitle(permissionDTO.getPermissionTitle());
        rolePermissionDTO.setPermissionType(permissionDTO.getPermissionType());
        rolePermissionDTO.setPermissionUrl(permissionDTO.getPermissionUrl());
        rolePermissionDTO.setPermissionClientId(permissionDTO.getPermissionClientId());
        rolePermissionDTO.setPermissionClientName(permissionDTO.getPermissionClientName());
        rolePermissionDTO.setIsPublic(permissionDTO.getIsPublic());
        return rolePermissionDTO;
    }

    @Override
    public List<RolePermissionDTO> listByRoleId(String roleId) {
        List<RolePermissionDTO> rolePermissionDTOs = rolePermissionDao.listByRoleId(roleId);
        if (rolePermissionDTOs.isEmpty()) {
            return rolePermissionDTOs;
        }
        List<String> permissionIds = new ArrayList<>();
        for (RolePermissionDTO rolePermissionDTO : rolePermissionDTOs) {
            permissionIds.add(rolePermissionDTO.getPermissionId());
        }
        List<PermissionDTO> permissionDTOs = permissionService.listByIds(permissionIds);
        if (permissionDTOs.isEmpty()) {
            return new ArrayList<>();
        }
        for (int i = 0; i < rolePermissionDTOs.size(); i++) {
            RolePermissionDTO rolePermissionDTO = rolePermissionDTOs.get(i);
            boolean isExist = false;
            for (PermissionDTO permissionDTO : permissionDTOs) {
                if (StringUtils.equals(rolePermissionDTO.getPermissionId(), permissionDTO.getPermissionId())) {
                    isExist = true;
                    rolePermissionDTO.setPermissionTitle(permissionDTO.getPermissionTitle());
                    rolePermissionDTO.setPermissionType(permissionDTO.getPermissionType());
                    rolePermissionDTO.setPermissionUrl(permissionDTO.getPermissionUrl());
                    rolePermissionDTO.setPermissionClientId(permissionDTO.getPermissionClientId());
                    rolePermissionDTO.setPermissionClientName(permissionDTO.getPermissionClientName());
                    rolePermissionDTO.setIsPublic(permissionDTO.getIsPublic());
                    break;
                }
            }
            if (!isExist) {
                rolePermissionDTOs.remove(i);
                i--;
            }
        }
        return rolePermissionDTOs;
    }

}
