package ink.wgink.service.user.service;

import ink.wgink.interfaces.user.IUserBaseService;
import ink.wgink.interfaces.user.IUserCheckService;
import ink.wgink.service.user.pojo.vos.RestPasswordVO;
import ink.wgink.service.user.pojo.vos.UpdateUsernameVO;
import ink.wgink.service.user.pojo.vos.UserVO;
import ink.wgink.pojo.result.UploadExcelResultDTO;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IUserService
 * @Description: 用户
 * @Author: wanggeng
 * @Date: 2021/1/25 3:06 下午
 * @Version: 1.0
 */
public interface IUserService extends IUserBaseService, IUserCheckService {

    /**
     * 保存用户
     *
     * @param userVO
     * @return
     */
    void save(UserVO userVO);

    /**
     * 注册用户
     *
     * @param userVO
     * @return
     */
    void register(UserVO userVO);

    /**
     * 删除用户
     *
     * @param ids id数组
     * @return
     */
    void remove(List<String> ids);

    /**
     * 修改用户
     *
     * @param userId 用户ID
     * @param userVO 用户内容
     * @return
     */
    void update(String userId, UserVO userVO);

    /**
     * 重置用户密码
     *
     * @param userId         用户ID
     * @param restPasswordVO 重置密码
     * @return
     */
    void resetPassword(String userId, RestPasswordVO restPasswordVO);

    /**
     * 修改用户名
     *
     * @param userId           用户ID
     * @param updateUsernameVO 更新用户名
     */
    void updateUsername(String userId, UpdateUsernameVO updateUsernameVO);

    /**
     * 导入Excel
     *
     * @param excel
     * @return
     * @throws IOException
     */
    UploadExcelResultDTO importExcel(MultipartFile excel) throws IOException;

}
